TASKS = [
    'style_transfer',
    'summarization',
    'dialog',
    'factual_consistency'
]

ALIGNS = [
    "E-bert",
    "E-roberta",
    "E-roberta-mnli",
    "D-topical_chat",
    "D-persona_chat",
    "D-cnndm",
    "D-xsum",
    "D-yelp",
    "D-mix-albert",
    "R-topical_chat",
    "R-persona_chat",
    "R-cnndm",
    "R-xsum",
    "R-yelp"
]

E_MODEL_CONFIGS = {
    'roberta': {'model_type': 'roberta-large'},
    'bert': {'model_type': 'bert-base-uncased'},
    'roberta-mnli': {'model_type': 'roberta-large-mnli', 'num_layers': 9}
}

DR_MODEL_LINKS = {
    "D-topical_chat": {
        "fact_to_response": "https://drive.google.com/file/d/1n91TuO1TFsM5ksVuclE2h6tBrHATSgC4/view?usp=sharing",
        "fact_history_to_response": "https://drive.google.com/file/d/1wLZEPd5K4H-F_wAAjIjQwyXDrzPO9TIg/view?usp=sharing"
    },
    "D-persona_chat": {
        "fact_to_response": "https://drive.google.com/file/d/1hqF8x4UoPvtCdzTus04UiNVFAZBN32Ak/view?usp=sharing",
        "fact_history_to_response": "https://drive.google.com/file/d/1mgtJKxtSCUNqJ1PITVtjLTXpve4woemf/view?usp=sharing"
    },
    "D-cnndm": {
        "doc_to_summ": "https://drive.google.com/file/d/1vahFzi2k74KRtQ6dxXzE5vYLQkartO9S/view?usp=sharing",
        "summ_to_ref": "https://drive.google.com/file/d/1G7-_RDeHNt2vzouK_sUkFRRvepmWiXID/view?usp=sharing"
    },
    "D-xsum": {
        "doc_to_summ": "https://drive.google.com/file/d/11y1FzAb0XcyDi2yDZXWDB3kpfc7BEXK8/view?usp=sharing"
    },
    "D-yelp": {
        "sent_to_sent": "https://drive.google.com/file/d/1Fw9Nibmu8-VhdwhPBJSoICgcnqmrxf6C/view?usp=sharing"
    },
    "D-mix-albert": {
        "doc_to_summ": "https://drive.google.com/file/d/1sEhsnMpZnv2dfZKCoEIP3e7jYTvNXgDp/view?usp=sharing"
    },
    
    "R-topical_chat": {
        "fact_to_response": "https://drive.google.com/file/d/1viv8deMc0HS2GPfUwCZDhacr0LAkjWrm/view?usp=sharing",
        "fact_history_to_response": "https://drive.google.com/file/d/1ThFlhvcUyuJtGZh3B9M42W9d6-YEjQmG/view?usp=sharing"
    },
    "R-persona_chat": {
        "fact_to_response": "https://drive.google.com/file/d/1VuaAZbik8XuosPOjsBzbiaYl3zv-3Iuq/view?usp=sharing",
        "fact_history_to_response": "https://drive.google.com/file/d/1VAEKwzkJNxWCaRqLlSu5eoq1TBvqHMqN/view?usp=sharing"
    },
    "R-cnndm": {
        "doc_to_summ": "https://drive.google.com/file/d/1gJ_YcDxz880mCtctOagxRMihBU-HFniS/view?usp=sharing",
        "summ_to_ref": "https://drive.google.com/file/d/16y2ZhBzSVUHNnTEN_vHc_9J5pzAVonKI/view?usp=sharing"
    },
    "R-xsum": {
        "doc_to_summ": "https://drive.google.com/file/d/1-nf7ob2GjR7kbCjhZtIIDB6kT7EY8pHx/view?usp=sharing"
    },
    "R-yelp": {
        "sent_to_sent": "https://drive.google.com/file/d/1mcXvNzeLyTTIEHRsUPmMSB8xvuLXcTrJ/view?usp=sharing"
    }
}



from ctc_score.scorer import Scorer


class FactualConsistencyScorer(Scorer):
    def __init__(self, align, aggr_type='mean', device='cuda'):
        Scorer.__init__(self, align=align, aggr_type=aggr_type, device=device)

    def score(self, grounding, hypo, aspect='consistency', remove_stopwords=False):
        kwargs = dict(
            grounding=grounding,
            hypo=hypo,
            remove_stopwords=remove_stopwords)

        if aspect == 'consistency':
            return self.score_consistency(**kwargs)
        else:
            raise NotImplementedError

    def score_consistency(self, grounding, hypo, remove_stopwords):
        aligner = self._get_aligner('doc_to_summ')

        return aligner.get_score(
            context=grounding,
            input_text=hypo,
            remove_stopwords=remove_stopwords)


from collections import defaultdict

import torch

from bert_score import BERTScorer
from bert_score.utils import get_bert_embedding, sent_encode

from ctc_score.models.aligner import Aligner


class BERTAligner(BERTScorer, Aligner):
    def __init__(self, aggr_type, *args, **kwargs):
        Aligner.__init__(self, aggr_type=aggr_type)
        BERTScorer.__init__(self, *args, **kwargs)

    def get_sim_matrix(self, candidate, reference):
        assert isinstance(candidate, str)
        assert isinstance(reference, str)

        idf_dict = defaultdict(lambda: 1.0)
        idf_dict[self._tokenizer.sep_token_id] = 0
        idf_dict[self._tokenizer.cls_token_id] = 0

        hyp_embedding, masks, padded_idf = get_bert_embedding(
            [candidate], self._model, self._tokenizer, idf_dict,
            device=self.device, all_layers=False,
        )
        ref_embedding, masks, padded_idf = get_bert_embedding(
            [reference], self._model, self._tokenizer, idf_dict,
            device=self.device, all_layers=False,
        )
        ref_embedding.div_(torch.norm(ref_embedding, dim=-1).unsqueeze(-1))
        hyp_embedding.div_(torch.norm(hyp_embedding, dim=-1).unsqueeze(-1))
        sim = torch.bmm(hyp_embedding, ref_embedding.transpose(1, 2))
        sim = sim.squeeze(0).cpu()

        r_tokens = [self._tokenizer.decode([i])
                    for i in sent_encode(self._tokenizer, reference)][1:-1]
        h_tokens = [self._tokenizer.decode([i])
                    for i in sent_encode(self._tokenizer, candidate)][1:-1]
        sim = sim[1:-1, 1:-1]

        if self.rescale_with_baseline:
            sim = (sim - self.baseline_vals[2].item()) / \
                  (1 - self.baseline_vals[2].item())

        return sim, r_tokens, h_tokens

    def align(self, input_text, context):
        mat, context_tokens, input_tokens = self.get_sim_matrix(
            candidate=input_text, reference=context)

        preds = torch.max(mat, dim=-1).values.tolist()

        return input_tokens, preds



import json
import random
import cleantext
from collections import namedtuple
from nltk import sent_tokenize
from datasets import load_dataset

from torch.utils.data import DataLoader


TokenClassificationExample = namedtuple('TokenClassificationExample', [
    'context', 'input_text', 'labels'])


TestExample = namedtuple('TestExample', ['system', 'input_text', 'context', 'score'])


def get_dataloaders(dataset, batch_size, num_workers, shuffle, collate_fn):
    if collate_fn == 'raw':
        collate_fn = lambda raw_batch: raw_batch

    return {split: DataLoader(
        dataset=dataset[split],
        batch_size=batch_size,
        collate_fn=collate_fn,
        shuffle=shuffle,
        num_workers=num_workers)
        for split in ['train', 'dev']}


def text_clean(text, remove_linefeed=False):
    text = text.replace('“', '"').replace('”', '"').replace(
        '’', '\'').replace('‘', '\'')

    if remove_linefeed:
        text = cleantext.clean(text, lowercase=True, extra_spaces=True)
    else:
        text = text.replace('\n', '<newline>')
        text = cleantext.clean(text, lowercase=True, extra_spaces=True)
        text = text.replace('<newline>', '\n')

    return text.strip()


def get_words(sent):
    return sent.split()


def get_discriminative_token_labels(template, answers, fillings):
    template_frags = template.split('<mask>')

    assert len(template_frags) == len(answers) + 1 == len(fillings) + 1

    hallu_frags = []
    for template_frag, answer, filling in zip(
            template_frags, answers, fillings):
        filling_label = int(filling.lower().strip() != answer.lower().strip())

        hallu_frags.append([template_frag.strip(), 0])
        hallu_frags.append([filling.strip(), filling_label])

    hallu_frags.append([template_frags[-1].strip(), 0])

    hallu_text = ' '.join([frag[0] for frag in hallu_frags]).strip()

    hallu_labels = []
    for i, hallu_frag in enumerate(hallu_frags):
        target_len = len(get_words(
            ' '.join([frag[0] for frag in hallu_frags[:i + 1]]).strip()))

        hallu_labels.extend([hallu_frag[1]] * (target_len - len(hallu_labels)))

    return hallu_text, hallu_labels


def get_context(constructed_doc, dataset_name, dialog_context):
    if dataset_name in ['xsum', 'cnndm']:
        context = constructed_doc['src']
    elif dataset_name in ['cnndm_ref']:
        context = constructed_doc['ref']
    elif dataset_name in ['yelp']:
        context = constructed_doc['text']
    elif dataset_name in ['persona_chat', 'topical_chat',
                          'persona_chat_fact', 'topical_chat_fact',]:
        if dataset_name == 'persona_chat':
            fact = '\n'.join([f'your persona: {t}'
                              for t in sent_tokenize(constructed_doc['fact'])])
            history = constructed_doc['history'].replace('<|endoftext|>', '\n')
        elif dataset_name == 'persona_chat_fact': 
            fact = '\n'.join([f'your persona: {t}'
                              for t in sent_tokenize(constructed_doc['fact'])])
        elif dataset_name == 'topical_chat':
            if constructed_doc['fact'] == '':
                fact = 'nofact\n'
            else:
                fact = '\n'.join([
                    f'fact: {t}' for t in constructed_doc['fact'].split('\n')])
            history = constructed_doc['history']
        elif dataset_name == 'topical_chat_fact':
            if constructed_doc['fact'] == '':
                fact = 'nofact\n'
            else:
                fact = '\n'.join([
                    f'fact: {t}' for t in constructed_doc['fact'].split('\n')])

        if dialog_context == 'fact_history':
            context = '\n\n\n'.join([fact.strip(), history.strip()])
        elif dialog_context == 'history_fact':
            context = '\n\n\n'.join([history.strip(), fact.strip()])
        elif dialog_context == 'fact':
            context = fact
        elif dialog_context == 'history':
            context = history
        else:
            raise ValueError

    return context


def get_examples_for_discriminative_construction(dataset_name):
    examples = []

    if dataset_name == 'xsum':
        for d in load_dataset('xsum')['train']:
            examples.append({
                'idx': len(examples),
                'src': text_clean(d['document']),
                'ref': text_clean(d['summary'])
            })
    elif dataset_name == 'cnndm' or dataset_name == 'cnndm_ref':
        for d in load_dataset('cnn_dailymail', '3.0.0')['train']:
            examples.append({
                'idx': len(examples),
                'src': text_clean(d['article']),
                'ref': text_clean(d['highlights'])
            })
    elif dataset_name == 'yelp':
        random.seed(159)
        for line in open('data/yelp/sentiment.train.0').readlines() + \
                    open('data/yelp/sentiment.train.1').readlines():
            if line.strip() != '':
                examples.append({
                    'idx': len(examples),
                    'text': text_clean(line.strip())
                })
        random.shuffle(examples)
    elif dataset_name == 'persona_chat' or dataset_name == 'persona_chat_fact':
        for d in load_dataset("bavard/personachat_truecased")['train']:
            examples.append({
                'idx': len(examples),
                'history': text_clean('\n'.join(d['history'])),
                'fact': text_clean('\n'.join(d['personality'])),
                'ref': text_clean(d['candidates'][-1])
            })
    elif dataset_name == 'topical_chat' or dataset_name == 'topical_chat_fact':
        for d in json.load(open('data/topical_chat/dialogs.json')):
            examples.append({
                'idx': len(examples),
                'history': text_clean(d['history']),
                'fact': text_clean(d['fact']),
                'ref': text_clean(d['response'])
            })

    return examples


def get_test_examples(dataset_name, aspect, dialog_context, n_references):
    raw_examples = json.load(open(f'data/{dataset_name}.json'))

    if dataset_name in ['qags_cnndm', 'qags_xsum', 'summeval']:
        examples = []
        for raw_example in raw_examples:
            if aspect == 'consistency':
                examples.append(TestExample(
                    system=raw_example['system'],
                    context=raw_example['document'],
                    input_text=raw_example['summary'],
                    score=raw_example[aspect]))

            elif aspect == 'relevance':
                example = [
                    TestExample(
                        system=raw_example['system'],
                        context=raw_example['document'],
                        input_text=raw_example['summary'],
                        score=raw_example[aspect]),
                    TestExample(
                        system=raw_example['system'],
                        context=raw_example['summary'],
                        input_text=raw_example['references'][:n_references],
                        score=raw_example[aspect])
                ]

                examples.append(example)

    elif dataset_name in ['persona_chat', 'topical_chat']:
        examples = []
        for raw_example in raw_examples:
            if dataset_name == 'topical_chat':
                fact = 'fact: ' + raw_example['fact']
            else:
                fact = raw_example['fact']
            history = raw_example['dialog_history']

            if dialog_context == 'fact':
                context = fact
            elif dialog_context == 'history':
                context = history
            elif dialog_context == 'fact_history':
                context = '\n\n\n'.join([fact.strip(), history.strip()])
            elif dialog_context == 'history_fact':
                context = '\n\n\n'.join([history.strip(), fact.strip()])

            examples.append(TestExample(
                system=raw_example['model'],
                context=context,
                input_text=raw_example['response'],
                score=raw_example[aspect]))

    elif dataset_name in ['yelp']:
        examples = []
        for raw_example in raw_examples:
            if aspect == 'preservation':
                example = [
                    TestExample(
                        system=raw_example['model'],
                        context=raw_example['input_sent'],
                        input_text=raw_example['output_sent'],
                        score=raw_example[aspect]),
                    TestExample(
                        system=raw_example['model'],
                        context=raw_example['output_sent'],
                        input_text=raw_example['input_sent'],
                        score=raw_example[aspect])
                ]

                examples.append(example)

    return examples


from collections import Counter
import os
from typing import List

import torch
from torch import nn

from ctc_score.models.aligner import Aligner

from transformers import AutoTokenizer, RobertaModel, AutoModel

from ctc_score.data_utils.data_utils import TokenClassificationExample, \
    text_clean, get_words

MAX_LENGTH = 512


class DiscriminativeAligner(Aligner, nn.Module):
    def __init__(self, aggr_type, model='roberta-large', device='cuda'):
        assert model in ['roberta-large', 'tals/albert-xlarge-vitaminc-mnli']
        Aligner.__init__(self, aggr_type)
        nn.Module.__init__(self)
        if model == 'roberta-large': 
            self._roberta = RobertaModel.from_pretrained('roberta-large')
            self._classifier = nn.Linear(self._roberta.config.hidden_size, 2)
        else:
            self._model = AutoModel.from_pretrained(model)
            self._classifier = nn.Linear(self._model.config.hidden_size, 2)
        self._tokenizer = AutoTokenizer.from_pretrained(model)
        self._space_char = '▁' if 'albert' in model else 'Ġ'
        self._bos_id = self._tokenizer.convert_tokens_to_ids(self._tokenizer.bos_token)
        

        self._hparams = None
        self._device = device

    def forward(self, input_text, words, context):
        if len(self._tokenizer(input_text)['input_ids']) > MAX_LENGTH:
            print('Length of input text exceeds max length! Skipping')
            return None
        
        tokens = (self._tokenizer(input_text, 
                                  context, 
                                  truncation='only_second',
                                  max_length=MAX_LENGTH,
                                  return_tensors='pt')
                  .to(self._device))
        if self._roberta: self._model = self._roberta
        features = self._model(**tokens).last_hidden_state[0]

        try:
            len_input_text = len(
                self._tokenizer(input_text)['input_ids'])
            word_features = self.extract_features_aligned_to_words(
                input_text=input_text,
                words=words,
                features=features[:len_input_text])
        except:
            print(f'Error bpe-to-words, '
                  f'word_logits=None for this batch: {input_text}')
            return None

        word_logits = self._classifier(word_features)

        return word_logits

    def extract_features_aligned_to_words(self, input_text, words, features):
        bpe_toks = self._tokenizer(
            input_text, return_tensors='pt')['input_ids'][0]

        alignment = self.align_bpe_to_words(bpe_toks, words)
        aligned_feats = self.align_features_to_words(features, alignment)

        assert len(words) + 2 == len(aligned_feats), 'words length {} does not match feature lenth {}. '.format(
            len(words), len(aligned_feats))

        return aligned_feats[1:-1]

    def align_features_to_words(self, features, alignment):
        """
        Align given features to words.

        Args:
            roberta (RobertaHubInterface): RoBERTa instance
            features (torch.Tensor): features to align of shape `(T_bpe x C)`
            alignment: alignment between BPE tokens and words returned by
                func:`align_bpe_to_words`.
        """
        assert features.dim() == 2

        bpe_counts = Counter(
            j for bpe_indices in alignment for j in bpe_indices)
        assert bpe_counts[0] == 0  # <s> shouldn't be aligned
        denom = features.new([bpe_counts.get(j, 1)
                             for j in range(len(features))])
        weighted_features = features / denom.unsqueeze(-1)

        output = [weighted_features[0]]
        largest_j = -1
        for bpe_indices in alignment:
            output.append(weighted_features[bpe_indices].sum(dim=0))
            largest_j = max(largest_j, *bpe_indices)
        for j in range(largest_j + 1, len(features)):
            output.append(weighted_features[j])
        output = torch.stack(output)
        # assert torch.all(torch.abs(output.sum(dim=0) - features.sum(dim=0)) < 1e-4)
        return output

    def align_bpe_to_words(self, bpe_tokens: torch.LongTensor, other_tokens: List[str]):
        """
        Helper to align GPT-2 BPE to other tokenization formats (e.g., spaCy).

        Args:
            roberta (RobertaHubInterface): RoBERTa instance
            bpe_tokens (torch.LongTensor): GPT-2 BPE tokens of shape `(T_bpe)`
            other_tokens (List[str]): other tokens of shape `(T_words)`

        Returns:
            List[str]: mapping from *other_tokens* to corresponding *bpe_tokens*.
        """
        assert bpe_tokens.dim() == 1
        assert bpe_tokens[0] == self._bos_id

        def clean(text):
            return text.strip()
        
        orig_bpe_toks = self._tokenizer.convert_ids_to_tokens(bpe_tokens)[1:-1]
        
        alignment = []
        bpe_indices = []
        for i, tok in enumerate(orig_bpe_toks): 
            if tok.startswith(self._space_char) and len(bpe_indices) > 0: 
                alignment.append(bpe_indices)
                bpe_indices = [i+1]
            else: 
                bpe_indices.append(i+1)
        alignment.append(bpe_indices)
        return alignment

    def align(self, input_text, context):
        input_text, context = text_clean(input_text), text_clean(context)

        input_text = self._tokenizer.decode(
            self._tokenizer(input_text, return_tensors='pt')['input_ids'][0][:MAX_LENGTH // 2])

        
        input_text = (input_text
                      .replace(self._tokenizer.bos_token, '')
                      .replace(self._tokenizer.eos_token, '')
                      .replace(self._tokenizer.unk_token, '')
                      .replace('  ', ' ')
                      .strip(' '))
        word_logits = self(
            input_text=input_text,
            words=get_words(input_text),
            context=context)

        if word_logits is None:
            return None, None

        preds = torch.softmax(word_logits, dim=-1)[:, 0]

        return get_words(input_text), preds.tolist()



from ctc_score.models.aligner import Aligner
import torch
import torch.nn as nn
import transformers
from transformers import AutoTokenizer

DEFAULT_TOKENIZER = 'bert-base-uncased'
MAX_LENGTH = 512

class BleurtModel(nn.Module):
    """ 
    We converted our BLEURT model from TF to PT according to the routine from 
    https://github.com/huggingface/datasets/issues/224#issuecomment-911782388, 
    and serve it with the following PT module, also from the same source. 
    """

    def __init__(self, config):
        super().__init__()
        self.config = config
        self.bert = transformers.BertModel(config)
        self.dense = nn.Linear(config.hidden_size,1)
    
    def forward(self, input_ids, input_mask, segment_ids):
        cls_state = self.bert(input_ids, input_mask, 
                              segment_ids).pooler_output
        return self.dense(cls_state)


class BLEURTAligner(Aligner): 
    def __init__(self, aggr_type, checkpoint, device, *args, **kwargs):
        Aligner.__init__(self, aggr_type=None)
        
        state_dict = torch.load(checkpoint)
        config = transformers.BertConfig()
        bleurt_model = BleurtModel(config)
        bleurt_model.load_state_dict(state_dict, strict=False)
        for param in bleurt_model.parameters():
            param.requires_grad = False
        bleurt_model.eval()
        
        self.device = device
        if self.device is None: 
            self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        
        self.bleurt_model = bleurt_model.to(self.device)
        self.tokenizer = AutoTokenizer.from_pretrained(DEFAULT_TOKENIZER)
        
        
    def align(self, input_text, context):
        encoding = (self.tokenizer(context, 
                                   input_text, 
                                   truncation='longest_first', 
                                   max_length=MAX_LENGTH, 
                                   return_tensors='pt')
                    .to(self.device))
        input_ids, input_mask, segment_ids = (encoding['input_ids'], 
                                              encoding['attention_mask'], 
                                              encoding['token_type_ids'])
        score = self.bleurt_model(input_ids, input_mask, segment_ids).tolist()[0]
        tokens = input_text.split()
        
        return tokens, score


import os

from ctc_score.download import maybe_download
from ctc_score.configs import ALIGNS, E_MODEL_CONFIGS, DR_MODEL_LINKS

from ctc_score.models.discriminative_aligner import DiscriminativeAligner
from ctc_score.models.bert_aligner import BERTAligner
from ctc_score.models.bleurt_aligner_pt import BLEURTAligner
import torch


class Scorer:
    def __init__(self, align, aggr_type, device):
        assert align in ALIGNS

        self._align = align
        self._aligners = {}

        self._aggr_type = aggr_type
        self._device = device

    def score(self, *args, **kwargs):
        raise NotImplementedError

    def _get_aligner(self, aligner_name):
        if aligner_name in self._aligners:
            return self._aligners[aligner_name]

        if self._align.startswith('E'):
            aligner = BERTAligner(
                **E_MODEL_CONFIGS[self._align[2:]],
                aggr_type=self._aggr_type,
                lang='en',
                device=self._device)
        else:
            aligner_link = DR_MODEL_LINKS[self._align][aligner_name]

            os.makedirs(
                f'{os.getenv("HOME")}/.cache/ctc_score_models/{self._align}/',
                exist_ok=True)
            maybe_download(
                urls=aligner_link,
                path=f'{os.getenv("HOME")}/.cache/',
                filenames=f'ctc_score_models/{self._align}/{aligner_name}.ckpt',
                num_gdrive_retries=3)
            ckpt_path = f'{os.getenv("HOME")}/.cache/' \
                        f'ctc_score_models/{self._align}/{aligner_name}.ckpt'

            if self._align.startswith('D'):
                model = 'tals/albert-xlarge-vitaminc-mnli' if 'albert' in self._align else 'roberta-large'
                
                aligner = DiscriminativeAligner(
                    aggr_type=self._aggr_type,
                    model=model, device=self._device).to(self._device)
                aligner.load_state_dict(torch.load(ckpt_path))
                aligner.eval()
            else:
                assert self._align.startswith('R')
                aligner = BLEURTAligner(
                    aggr_type=self._aggr_type,
                    checkpoint=ckpt_path,
                    device=self._device)

        self._aligners[aligner_name] = aligner
        return self._aligners[aligner_name]



from ctc_score.scorer import Scorer


class SummarizationScorer(Scorer):
    def __init__(self, align, aggr_type='mean', device='cuda'):
        Scorer.__init__(self, align=align, aggr_type=aggr_type, device=device)

    def score(self, doc, refs, hypo, aspect, remove_stopwords=False):
        kwargs = dict(
            doc=doc,
            refs=refs,
            hypo=hypo,
            remove_stopwords=remove_stopwords)

        if aspect == 'consistency':
            return self.score_consistency(**kwargs)
        elif aspect == 'relevance':
            return self.score_relevance(**kwargs)
        else:
            raise NotImplementedError

    def score_consistency(self, doc, refs, hypo, remove_stopwords):
        aligner = self._get_aligner('doc_to_summ')

        return aligner.get_score(
            context=doc,
            input_text=hypo,
            remove_stopwords=remove_stopwords)

    def score_relevance(self, doc, refs, hypo, remove_stopwords):
        aligner_doc_summ = self._get_aligner('doc_to_summ')
        align_y_x = aligner_doc_summ.get_score(
            context=doc,
            input_text=hypo,
            remove_stopwords=remove_stopwords)

        aligner_summ_ref = self._get_aligner('summ_to_ref')
        align_r_y = []
        for ref in refs:
            align_r_y.append(aligner_summ_ref.get_score(
                context=hypo,
                input_text=ref,
                remove_stopwords=remove_stopwords))
        align_r_y = sum(align_r_y) / len(align_r_y)

        return align_y_x * align_r_y



import sys
import setuptools

if sys.version_info < (3, 6):
    sys.exit('Python>=3.6 is required by ctc_score.')

setuptools.setup(
    name="ctc_score",
    version='0.1.2',
    url="https://github.com/tanyuqian/ctc-gen-eval",
    author="Mingkai Deng*, Bowen Tan*, Zhengzhong Liu, Eric P. Xing, Zhiting Hu, Yuheng Zha",
    description="CTC: A Unified Framework for Evaluating Natural Language Generation",
    long_description=open("README.md", "r", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    keywords='CTC Score',
    license='MIT',

    packages=setuptools.find_packages(),
    install_requires=[
        'nltk',
        'transformers>=4.3',
        'datasets>=1.5',
        'cleantext',
        'bert_score>=0.3',
    ],
    entry_points={
        'console_scripts': [
            "ctc_score=ctc_score_cli.score:main",
        ]
    },
    include_package_data=True,
    python_requires='>=3.6',
    classifiers=[
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
    ],
)



from ctc_score.scorer import Scorer


class DialogScorer(Scorer):
    def __init__(self, align, aggr_type='sum', device='cuda'):
        Scorer.__init__(self, align=align, aggr_type=aggr_type, device=device)

    def score(self, fact, dialog_history, hypo, aspect, remove_stopwords=True):
#         if 'topical_chat' in self._align:
#             fact = 'fact: ' + fact

        kwargs = dict(
            fact=fact,
            dialog_history=dialog_history,
            hypo=hypo,
            remove_stopwords=remove_stopwords)

        if aspect == 'groundedness':
            return self.score_groundedness(**kwargs)
        elif aspect == 'engagingness':
            return self.score_engagingness(**kwargs)
        else:
            raise NotImplementedError

    def score_groundedness(self, fact, dialog_history, hypo, remove_stopwords):
        context = fact
        aligner = self._get_aligner('fact_to_response')

        return aligner.get_score(
            context=context,
            input_text=hypo,
            remove_stopwords=remove_stopwords)

    def score_engagingness(self, fact, dialog_history, hypo, remove_stopwords):
        context = '\n\n\n'.join([fact.strip(), dialog_history.strip()])
        aligner = self._get_aligner('fact_history_to_response')

        return aligner.get_score(
            context=context,
            input_text=hypo,
            remove_stopwords=remove_stopwords)
